@@ -1,9 +1,10 @@
 /*
    energy.c: energy package
 
-   Author:  Maria Rizzo <rizzo@math.ohiou.edu>
+   Author:  Maria Rizzo <mrizzo @ bgnet.bgsu.edu>
    Created: 4 Jan 2004 for R-1.8.1
    Revised: 20 March 2004 (E2, twosampleIEtest added)
+   Revised: 13 June 2004 (distance() changed, some utilities added)
 
    mvnEstat()     computes the E-test of multivariate normality
    ksampleEtest() performs the multivariate E-test for equal distributions,
@@ -21,7 +22,7 @@ void   mvnEstat(double *y, int *byrow, int *nobs, int *dim, double *stat);
 void   poisMstat(int *x, int *nx, double *stat);
 void   ksampleEtest(double *x, int *byrow, int *nsamples, int *sizes, int *dim,
             int *R, double *e0, double *e, double *pval);
-void   twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN, 
+void   twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN,
                      int *R, double *e0, double *e, double *pval);
 void   E2sample(double *x, int *sizes, int *dim, double *stat);
 
@@ -34,7 +35,9 @@ void   distance(double **bxy, double **D, int N, int d);
 void   sumdist(double *x, int *byrow, int *nrow, int *ncol, double *lowersum);
 
 double **alloc_matrix(int r, int c);
+int    **alloc_int_matrix(int r, int c);
 void   free_matrix(double **matrix, int r, int c);
+void   free_int_matrix(int **matrix, int r, int c);
 void   permute(int *J, int n);
 void   roworder(double *x, int *byrow, int r, int c);
 void   vector2matrix(double *x, double **y, int N, int d, int isroworder);
@@ -270,15 +273,15 @@ void ksampleEtest(double *x, int *byrow,
 
 }
 
-void twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN, 
+void twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN,
                    int *R, double *e0, double *e, double *pval)
 {
     /*
       exported for R energy package: incomplete E test for equal distributions
-      x         the pooled sample 
+      x         the pooled sample
       *byrow    logical, TRUE if x is stored by row
                 pass x=as.double(t(x)) if *byrow==TRUE
-      sizes     vector of original sample sizes 
+      sizes     vector of original sample sizes
       *dim      dimension of data in x
       *iN       incomplete sample size
       *R        number of replicates for permutation test
@@ -290,11 +293,11 @@ void twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN,
     int    i, b, B = (*R), ek, nrow, ncol = (*dim), N = (*iN);
     int    *perm;
     double **data;
-    
+
     nrow = sizes[0] + sizes[1];
     I[0] = sizes[0] > N ? N : sizes[0];
     I[1] = sizes[1] > N ? N : sizes[1];
-    
+
     if (*byrow == FALSE)
         roworder(x, byrow, nrow, ncol);
     data = alloc_matrix(nrow, ncol);
@@ -303,7 +306,7 @@ void twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN,
     for (i=0; i<nrow; i++) perm[i] = i;
     start[0] = 0;
     start[1] = sizes[0];
-    permute(perm + start[0], sizes[0]);    
+    permute(perm + start[0], sizes[0]);
     permute(perm + start[1], sizes[1]);
 
     *e0 = E2(data, I, start, ncol, perm);
@@ -318,7 +321,7 @@ void twosampleIEtest(double *x, int *byrow, int *sizes, int *dim, int *iN,
     }
     Free(data);
     Free(perm);
-    return;       
+    return;
 }
 
 void sumdist(double *x, int *byrow, int *nrow, int *ncol, double *lowersum)
@@ -349,7 +352,7 @@ void sumdist(double *x, int *byrow, int *nrow, int *ncol, double *lowersum)
     (*lowersum) = sum;
 }
 
-double E2(double **x, int *sizes, int *start, int ncol, int *perm) 
+double E2(double **x, int *sizes, int *start, int ncol, int *perm)
 {
     int    m=sizes[0], n=sizes[1];
     int    row1=start[0], row2=start[1];
@@ -495,6 +498,18 @@ double **alloc_matrix(int r, int c)
     return matrix;
 }
 
+
+int **alloc_int_matrix(int r, int c)
+{
+    /* allocate an integer matrix with r rows and c columns */
+    int i;
+    int **matrix;
+    matrix = Calloc(r, int *);
+    for (i = 0; i < r; i++)
+    matrix[i] = Calloc(c, int);
+    return matrix;
+}
+
 void free_matrix(double **matrix, int r, int c)
 {
     /* free a matrix with r rows and c columns */
@@ -503,6 +518,14 @@ void free_matrix(double **matrix, int r, int c)
     Free(matrix);
 }
 
+void free_int_matrix(int **matrix, int r, int c)
+{
+    /* free an integer matrix with r rows and c columns */
+    int i;
+    for (i = 0; i < r; i++) Free(matrix[i]);
+    Free(matrix);
+}
+
 void permute(int *J, int n)
 {
     /*
@@ -539,7 +562,7 @@ void vector2matrix(double *x, double **y, int N, int d, int isroworder) {
 void distance(double **data, double **D, int N, int d) {
     /*
        compute the distance matrix of sample in N by d matrix data
-       like D <- as.matrix(dist(data)) in R
+       equivalent R code is:  D <- as.matrix(dist(data))
     */
     int    i, j, k;
     double dif;
@@ -580,4 +603,3 @@ void roworder(double *x, int *byrow, int r, int c) {
     *byrow = TRUE;
     return;
 }
-
